#!/System/Index/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
# === ClassDocuShower
#
# This file will extract the comment from a class and return a string about
# the docu. That comment must be padded inside lines with '# ====' lines,
# like this file here.
#
# Usage example:
#   ClassDocuShower.new('file_location_comes_in_here')
# =========================================================================== #
# require 'class_docu_shower.rb'
# =========================================================================== #
require 'opn'
begin
  require 'usher'
rescue LoadError; end
begin
  require 'colours'
rescue LoadError; end
require 'class_docu_shower/version/version.rb'
require 'class_docu_shower/colours.rb'

class ClassDocuShower # === ClassDocuShower.new

  include Colours

  N = "\n"

  # ========================================================================= #
  # === DEFAULT_FILE
  # ========================================================================= #
  DEFAULT_FILE =
    '/Users/x/DATA/PROGRAMMING_LANGUAGES/RUBY/src/tools/show_available_users.rb'

  # ========================================================================= #
  # === FORBIDDEN_CHARACTERS
  # ========================================================================= #
  FORBIDDEN_CHARACTERS = '# ===='

  # ========================================================================= #
  # === ENCODING_TO_USE
  # ========================================================================= #
  ENCODING_TO_USE = 'ISO-8859-1' # Which Encoding to consistently use for this class.

  # ========================================================================= #
  # === initialize
  #
  # The first argument should be a .rb file.
  #
  # The second argument can be :silent, in which case we will be silent
  # rather than verbose.
  # ========================================================================= #
  def initialize(
      i           = nil,
      run_already = true
    )
    be_silent
    reset
    set_files(i)
    if run_already.is_a? Hash # Intercept hash here.
      if run_already.has_key? :verbose
        @be_verbose = run_already[:verbose]
        run_already = true
      end
    else
      case run_already
      when :silent
        be_silent
        run_already = true
      when :no_colours
        @use_colours = false
        run_already = true
      end
    end
    if block_given?
      yielded = yield
      case yielded
      when :do_not_report_the_filename
        @report_the_filename = false
      end
    end
    run if run_already
  end

  # ========================================================================= #
  # === reset                                                     (reset tag)
  # ========================================================================= #
  def reset
    @data   = nil
    if Object.const_defined? :Usher
      @usher = Usher.new
    else
      @usher = nil
    end
    @result = nil
    @use_colours = true
    @report_the_filename = true
  end

  # ========================================================================= #
  # === reset_usher_object
  # ========================================================================= #
  def reset_usher_object
    if @usher
      @usher.reset
      @usher.allow_n_changes = 1
    end
  end

  # ========================================================================= #
  # === be_verbose
  # ========================================================================= #
  def be_verbose
    @be_verbose = true
  end

  # ========================================================================= #
  # === be_silent
  # ========================================================================= #
  def be_silent
    @be_verbose = false
  end

  # ========================================================================= #
  # === file?
  # ========================================================================= #
  def file?
    @file.to_s
  end

  # ========================================================================= #
  # === show_the_result
  # ========================================================================= #
  def show_the_result
    e @result if result?
  end

  # ========================================================================= #
  # === use_colours?
  # ========================================================================= #
  def use_colours?
    @use_colours
  end

  # ========================================================================= #
  # === read_in_file
  #
  # We determine @data here.
  # ========================================================================= #
  def read_in_file(this_file = @file)
    if File.exist? this_file
      @data = File.readlines(this_file, :encoding => ENCODING_TO_USE)
    else # We should report.
      result = 'We could not find a file at '+sfile(this_file)+','
      result = Colours.remove_colours(result) unless use_colours?
      opn; e result
      result = 'thus can not check whether it has an documentation.'
      result = Colours.remove_colours(result) unless use_colours?
      opn; e result
    end
  end

  # ========================================================================= #
  # === result?
  # ========================================================================= #
  def result?
    @result
  end

  # ========================================================================= #
  # === show_result
  # ========================================================================= #
  def show_result
    if @be_verbose # We will be chatty here, aka, verbose.
      result = 'From the file '+sfile(file?)+' we '
      result = Colours.remove_colours(result) unless use_colours?
      opn; e result
      result = 'can show this result:'
      result = Colours.remove_colours(result) unless use_colours?
      opn; e result
    end 
    show_the_result
  end

  # ========================================================================= #
  # === files?
  # ========================================================================= #
  def files?
    @files
  end; alias files files? # === files

  # ========================================================================= #
  # === sfile
  # ========================================================================= #
  def sfile(i = '')
    i = Colours.sfile(i) if use_colours?
    return i
  end

  # ========================================================================= #
  # === process
  #
  # This method contains the in-class logic.
  # ========================================================================= #
  def process
    @files.each {|file|
      show_file_header(file)
      read_in_file(file)
      process_dataset
      show_result
      e
    }
  end

  # ========================================================================= #
  # === process_dataset
  # ========================================================================= #
  def process_dataset
    reset_usher_object
    if @data
      @data.each {|entry| # @data is an Array.
        if entry.include? FORBIDDEN_CHARACTERS # Reject leading '# ===' parts.
          @usher.toggle if @usher
        else
          # ================================================================= #
          # Else add the entry. We should not use .lstrip on it because
          # that way we can preserve the original indent.
          # ================================================================= #
          entry.delete!('#') # .lstrip
          if use_colours? and Object.const_defined?(:Colours)
            if entry.start_with? ' ==='
              entry = orange(entry)
            elsif entry.include? 'Usage examples:'
              entry = lightblue(entry)
            end
          end
          @usher << entry if @usher
        end
      }
      @result = @usher._ if @usher
    end
  end

  # ========================================================================= #
  # === set_files
  # ========================================================================= #
  def set_files(i = DEFAULT_FILE)
    i = DEFAULT_FILE if i.nil?
    # i = i.to_s.dup if i
    i = [i] unless i.is_a? Array # Must always be an Array.
    i.flatten! # But keep it flattened.
    @files = i
  end

  # ========================================================================= #
  # === show_file_header
  #
  # This method will show the header, with an introduction like
  # "From file bla".
  # ========================================================================= #
  def show_file_header(i)
    if @report_the_filename#
      result = 'From file '+sfile(i)+':'+N+N
      result = Colours.remove_colours(result) unless use_colours?
      e result
    end
  end

  # ========================================================================= #
  # === run                                                         (run tag)
  # ========================================================================= #
  def run
    process
  end

  # ========================================================================= #
  # === ClassDocuShower[]
  # ========================================================================= #
  def self.[](i)
    ClassDocuShower.new(i)
  end

end

if __FILE__ == $PROGRAM_NAME
  _ = ClassDocuShower.new(ARGV, :silent)
  # pp _.result? # This here for debug.
end # rdocu